@@ -9,11 +9,13 @@ from .base import RemoteStorageBackendMixin
 
 class StorageBackend(RemoteStorageBackendMixin, BaseStorageBackend):
     """A storage backend that uses Google Cloud Storage (GCS)."""
+    kind = 'google'
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.client = storage.Client()
         self.bucket = self.client.get_bucket(os.environ['GOOGLE_GCS_BUCKET'])
+        self.project = os.environ['GOOGLE_CLOUD_PROJECT']
 
     def close(self, handler):
         """Flush and close the IO object.
@@ -60,3 +62,12 @@ class StorageBackend(RemoteStorageBackendMixin, BaseStorageBackend):
         """Remove a path."""
         self.bucket.delete_blobs(
             blobs=list(self.bucket.list_blobs(prefix=self.storage_path(path))))
+
+    def update_labels(self, labels):
+        """Update dictionary `labels` with the label for a specific
+        storage backend.
+        """
+        labels.update({
+            'cloud.google.com/project': self.project,
+            'storage.googleapis.com/bucket': self.bucket
+        })
